from kid_sdk.kid_profile.models import DomesticProfile, InternationalProfile


__all__ = [
    "UserProfileMixin",
]


class UserProfileMixin:
    @property
    def profile(self):
        if not hasattr(self, "_profile"):
            self._profile = self._get_profile()
        return self._profile

    def _get_profile(self):
        try:
            return DomesticProfile.objects.using("domestic").get(smart_id=self.smart_id)
        except DomesticProfile.DoesNotExist:
            try:
                return InternationalProfile.objects.using("international").get(smart_id=self.smart_id)
            except InternationalProfile.DoesNotExist:
                return None

    @property
    def first_name(self):
        return self.profile.first_name if self.profile else None

    @property
    def last_name(self):
        return self.profile.last_name if self.profile else None

    @property
    def phone(self):
        return self.profile.phone if self.profile else None

    @property
    def email(self):
        return self.profile.email if self.profile else None

    @property
    def user_type(self):
        if not self.profile:
            return "unknown"
        return "domestic" if isinstance(self.profile, DomesticProfile) else "international"



import os

DEBUG = False

INSTALLED_APPS = [
    "kid_sdk.kid_profile",
]

DATABASES = {
    db: {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": db,
        "USER": os.environ.get(f"{db.upper()}_POSTGRES_USER", None),
        "PASSWORD": os.environ.get(f"{db.upper()}_POSTGRES_PASSWORD", None),
        "HOST": os.environ.get(f"{db.upper()}_POSTGRES_HOST", None),
        "PORT": os.environ.get(f"{db.upper()}_POSTGRES_PORT", None),
        "ATOMIC_REQUESTS": False,
    }
    for db in ["domestic", "international"]
}
DATABASES["default"] = {
    "ENGINE": "django.db.backends.sqlite3",
    "NAME": "profile",
}

DATABASE_ROUTERS = [
    "kid_sdk.kid_profile.routers.ProfileRouter",
]


# General
TEST_DOMAIN = "https://dev-smartid.zonesmart.ru"
DOMAIN = "https://id.kokocgroup.ru"

AUTHORIZATION_URL = "{domain}"
TOKEN_ISSUE_URL = "{domain}/api/oauth2/issue/"
GET_USER_DATA_URL = "{domain}/api/user/auth_data/"

DEFAULT_SCOPE = "smart_id first_name last_name email phone roles"



from django.db import models
from django.core.exceptions import FieldError

from kid_sdk.kid_profile.models import DomesticProfile, InternationalProfile


__all__ = [
    "ProfileManager",
]


class ProfileQuerySetException(Exception):
    pass


class ProfileQuerySet(models.query.QuerySet):
    FIELDS = ["first_name", "last_name", "phone", "email"]

    def _filter_domestic_ids(self, *args, filters: dict) -> list:
        q = models.Q()
        for field, value in filters.items():
            if field.split("__")[0] in self.FIELDS:
                q &= models.Q(**{field: value})
        return list(DomesticProfile.objects.using("domestic").filter(*args, q).values_list("smart_id", flat=True))

    def _filter_international_ids(self, *args, filters: dict) -> list:
        q = models.Q()
        for field, value in filters.items():
            if field.split("__")[0] in self.FIELDS:
                q &= models.Q(**{field: value})
        return list(
            InternationalProfile.objects.using("international").filter(*args, q).values_list("smart_id", flat=True)
        )

    def filter(self, *args, **filters):
        try:
            domestic_ids = self._filter_domestic_ids(*args, filters=filters)
            international_ids = self._filter_international_ids(*args, filters=filters)
        except FieldError:
            if args:
                allowed_fields = ", ".join(["smart_id"] + self.FIELDS)
                raise ProfileQuerySetException(
                    f"Использование Q функций разрешено только с полями из моделей Profile: {allowed_fields}"
                )
            raise

        all_ids = list(set(domestic_ids) | set(international_ids))
        local_filters = {k: v for k, v in filters.items() if k.split("__")[0] not in self.FIELDS}
        return super().filter(**local_filters, smart_id__in=all_ids)


class ProfileManager(models.Manager):
    def get_queryset(self):
        return ProfileQuerySet(self.model, using=self._db)

    def all(self):
        return ProfileQuerySet(self.model, using=self._db).all()



from django.contrib.auth import get_user_model


class ProfileRouter:
    def db_for_read(self, model, **hints):
        if model.__name__ in ["DomesticProfile", "InternationalProfile"]:
            return model.__name__.lower().replace("profile", "")
        return None

    def db_for_write(self, model, **hints):
        if model.__name__ in ["DomesticProfile", "InternationalProfile"]:
            return model.__name__.lower().replace("profile", "")
        return None

    def allow_relation(self, obj1, obj2, **hints):
        User = get_user_model()
        if isinstance(obj1, User) or isinstance(obj2, User):
            return True
        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        if model_name in ["domesticprofile", "internationalprofile"]:
            return db == model_name.replace("profile", "")
        return None


class ReadOnlyProfileRouter(ProfileRouter):
    def db_for_write(self, model, **hints):
        if model.__name__ in ["DomesticProfile", "InternationalProfile"]:
            return False
        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        if model_name in ["domesticprofile", "internationalprofile"]:
            return False
        return None



from django.db import models


__all__ = [
    "DomesticProfile",
    "InternationalProfile",
]


class Profile(models.Model):
    smart_id = models.UUIDField(unique=True, db_index=True, primary_key=True, max_length=128)
    first_name = models.CharField(verbose_name="Имя", max_length=128)
    last_name = models.CharField(verbose_name="Фамилия", max_length=128)
    email = models.EmailField(unique=True, verbose_name="E-mail", max_length=128)
    phone = models.CharField(unique=True, verbose_name="Телефон", max_length=128)

    class Meta:
        abstract = True


class DomesticProfile(Profile):
    class Meta:
        ordering = ["smart_id"]


class InternationalProfile(Profile):
    class Meta:
        ordering = ["smart_id"]



# Generated by Django 5.2.3 on 2025-06-27 08:52

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="DomesticProfile",
            fields=[
                (
                    "smart_id",
                    models.UUIDField(
                        db_index=True, primary_key=True, serialize=False, unique=True
                    ),
                ),
                ("first_name", models.CharField(max_length=128, verbose_name="Имя")),
                ("last_name", models.CharField(max_length=128, verbose_name="Фамилия")),
                (
                    "email",
                    models.EmailField(
                        max_length=128, unique=True, verbose_name="E-mail"
                    ),
                ),
                (
                    "phone",
                    models.CharField(
                        max_length=128, unique=True, verbose_name="Телефон"
                    ),
                ),
            ],
            options={
                "ordering": ["smart_id"],
            },
        ),
        migrations.CreateModel(
            name="InternationalProfile",
            fields=[
                (
                    "smart_id",
                    models.UUIDField(
                        db_index=True, primary_key=True, serialize=False, unique=True
                    ),
                ),
                ("first_name", models.CharField(max_length=128, verbose_name="Имя")),
                ("last_name", models.CharField(max_length=128, verbose_name="Фамилия")),
                (
                    "email",
                    models.EmailField(
                        max_length=128, unique=True, verbose_name="E-mail"
                    ),
                ),
                (
                    "phone",
                    models.CharField(
                        max_length=128, unique=True, verbose_name="Телефон"
                    ),
                ),
            ],
            options={
                "ordering": ["smart_id"],
            },
        ),
    ]



import logging

import pendulum
import requests

from authlib.integrations.requests_client import OAuth2Session
from authlib.jose import jwt
from authlib.jose.errors import (
    JoseError,
    DecodeError,
    BadSignatureError,
    ExpiredTokenError,
    InvalidClaimError,
)
from requests.exceptions import HTTPError

from .config import (
    AUTHORIZATION_URL,
    DEFAULT_SCOPE,
    DOMAIN,
    GET_USER_DATA_URL,
    TEST_DOMAIN,
    TOKEN_ISSUE_URL,
)
from .utils import validate_token


class KidOAuth2Client:
    def __init__(
        self,
        client_id: str,
        client_secret: str,
        redirect_uri: str,
        scope: str = DEFAULT_SCOPE,
        test: bool = False,
    ):
        """
        Initializes the OAuth2 client.

        :param client_id: Client identifier
        :param client_secret: Client secret
        :param redirect_uri: Redirect URI
        :param scope: A space-separated string of OAuth2 scopes defining the level of access
                      the client is requesting. Scopes control the permissions granted by the
                      authorization server, such as access to user profile, email, or other resources.
                      Example: "smart_id first_name last_name email phone"
        :param test: Boolean indicating whether to use the test domain
        """
        self.redirect_uri = redirect_uri
        self.session = OAuth2Session(
            client_id=client_id,
            client_secret=client_secret,
            redirect_uri=redirect_uri,
            scope=scope,
        )
        self.token = None
        self.token_expires_at = None
        self.domain = TEST_DOMAIN if test else DOMAIN
        self.authorization_url = AUTHORIZATION_URL.format(domain=self.domain)
        self.token_issue_url = TOKEN_ISSUE_URL.format(domain=self.domain)
        self.get_user_data_url = GET_USER_DATA_URL.format(domain=self.domain)

    @property
    def token_is_expired(self) -> bool:
        if self.token_expires_at is None:
            return True
        return pendulum.now("UTC").timestamp() >= self.token_expires_at

    def get_authorization_url(self) -> str:
        """
        Creates an authorization URL.

        :return: Authorization URL
        """
        return self.session.create_authorization_url(self.authorization_url, redirect_uri=self.redirect_uri)[0]

    def fetch_token(self, code: str) -> str:
        """
        Fetches the access token.

        :param code: Authorization code
        :return: Access token
        :raises Exception: If there is an error fetching the token
        """
        try:
            self.token = self.session.fetch_token(self.token_issue_url, code=code)
        except Exception as e:
            logging.error(f"Error fetching token: {e}")
            raise

        self.token_expires_at = self.token["expires_at"]
        return self.token["access_token"]

    @validate_token
    def get_user_data(self) -> dict:
        """
        Retrieves user data using the token.

        :return: Dictionary containing user data
        :raises HTTPError: If the request fails
        """
        self.session.token = self.token

        try:
            response = self.session.get(self.get_user_data_url)
            response.raise_for_status()
        except HTTPError as http_err:
            logging.error(f"HTTP error occurred: {http_err}")
            raise
        except Exception as err:
            logging.error(f"Error occurred: {err}")
            raise

        return response.json()


class KidJWTClientException(Exception):
    pass


class KidJWTClient:
    @staticmethod
    def parse_jwt(encoded_jwt: str, key: str) -> dict:
        """
        Parses a JWT token.

        :param encoded_jwt: Encoded JWT token
        :param key: Key for decoding
        :return: Dictionary containing claims or None if an error occurs
        """
        try:
            claims = jwt.decode(encoded_jwt, key)
            claims.validate()
        except DecodeError as e:
            message = f"Failed to decode JWT: {e}"
            logging.error(message)
            raise KidJWTClientException(message)
        except BadSignatureError as e:
            message = f"Invalid signature: {e}"
            logging.error(message)
            raise KidJWTClientException(message)
        except ExpiredTokenError as e:
            message = f"Token has expired: {e}"
            logging.error(message)
            raise KidJWTClientException(message)
        except InvalidClaimError as e:
            message = f"Invalid claim: {e}"
            logging.error(message)
            raise KidJWTClientException(message)
        except JoseError as e:
            message = f"Failed to decode JWT: {e}"
            logging.error(message)
            raise KidJWTClientException(message)

        if not isinstance(claims, dict):
            logging.error("Claims data is not a dictionary")
            raise TypeError("Claims data is not a dictionary")

        return claims

    @classmethod
    def mark_user_as_transferred(cls, smart_id: str, site_id: str, key: str, test: bool = False) -> dict:
        """
        Отправка данных пользователя в Kokoc ID
        """
        data = {
            "site": site_id,
            "token": jwt.encode(header={"alg": "HS256"}, payload={"smart_id": smart_id}, key=key).decode(),
        }
        url = TEST_DOMAIN if test else DOMAIN
        response = requests.post(url + "/api/user/mark_user_as_transferred/", json=data)
        if response.status_code != 201:
            return response.json()



from setuptools import setup, find_packages


setup(
    name="kid_sdk",
    version="1.0",
    author="Zonesmart",
    author_email="e.beliakov@dev.kokoc.com",
    packages=find_packages(),
    install_requires=[
        "authlib",
        "requests",
        "pendulum",
        "django>=4",
        "psycopg2==2.9.5",
    ],
)



import functools


def validate_token(func):
    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        if not self.token:
            raise Exception("Token is not available. Please fetch the token first.")
        if self.token_is_expired:
            raise Exception("Token has expired. Please fetch a new token.")
        return func(self, *args, **kwargs)

    return wrapper
